module Decoding (show1)
      
where
import Syntax

-------------------------------------------------------------------------------------------------------------------------------------------------------
-- the module Decoding provides decoding expressions of type Exp into a string
------------------------------------------------------------------------------------------------------------------------------------------------------- 

show1 :: Exp -> String
show1 x = case x of 
           (V s) -> s;
           (Lambda x e) -> "(\\" ++ x ++ " -> " ++ show1 e ++ ")";
           (Constr 6 3 2 xs) -> (show (decodeInt (Constr 6 3 2 xs)));
           (Constr 1 b c []) -> "[]";
           (Constr 2 2 0 []) -> "False";
           (Constr 2 1 c []) -> "True" ;
           (Constr a b c [x,y]) -> if a == 1 then  "[" ++ show1 x ++ "," ++show1 y++ "]" else "(" ++ show1 x++ "," ++show1 y++ ")";
           (Constr a b c [x,y,z]) -> "(" ++show x++ "," ++show y++ "," ++show z++")";
           (Constr a b c [x,y,z,w]) -> "(" ++show x++ "," ++show y++ "," ++show z++ "," ++show w++ ")";
            CCell -> "cell";
            CThread -> "thread";
            CHandle -> "handle";
            CLazy -> "lazy";
            CUnit -> "unit";
            CBuffer -> "buffer";
            CGet -> "get";
            (App e1 e2) -> "(" ++show1 e1 ++ " " ++ show1 e2 ++")";
            Case e alts -> "case " ++ show1 e ++  " of (" ++ show2 alts ++ ")";
            Exch a b -> "exch ("++ show a ++","++ show1 b++")";
            Put a b -> "put (" ++show1 a ++ "," ++ show1 b ++")";
            
            o -> error (show o)

show2 x = case x of
           (x:xs) -> show3 x ++ show2 xs 
           [] -> []              

show3 x = case x of 
          --   (Alt 6 3 2 xs e) -> (show (decodeInt (Constr 6 3 2 xs)));
            (Alt 1 b c [] e) -> "[] ->" ++ show1 e;
            (Alt 2 b c [] e) -> if b==1 then "True" else "False" ++ "->" ++ show1 e;
            (Alt a b c [x,y] e) -> if a == 1 then  "[" ++ show x ++ "," ++show y++ "]" else "(" ++ show x++ "," ++show y++ ")" ++ "->" ++ show1 e;
            (Alt a b c [x,y,z] e) -> "(" ++show x++ "," ++show y++ "," ++show z++")" ++ "->" ++ show1 e;
            (Alt a b c [x,y,z,w] e) -> "(" ++show x++ "," ++show y++ "," ++show z++ "," ++show w++ ")" ++ "->" ++ show1 e;


------------------------------------------------------------------------------------------------------------------------------------------------------
-- the subfunction decodeInt decodes an original view of integers (in decimal code) 
------------------------------------------------------------------------------------------------------------------------------------------------------

decodeInt :: Exp -> Int
decodeInt (Constr 6 3 2 (x:y:[])) = case x of 
                                    (Constr 2 1 0 []) -> (decodeInt2 y);
                                                   _  -> (-decodeInt2 y)

decodeInt2 :: Exp -> Int
decodeInt2 (Constr 1 2 2 (x:y:[])) =
		let a=(case x of {(Constr 6 1 0 [])->0;_->1})
		    b=(case y of {(Constr 1 2 2 ls)->(decodeInt2 y);_->0})
		in a+b*2

--------------------------------------------------------------------------------------------------------------------------------------------------------
-- some tests
--------------------------------------------------------------------------------------------------------------------------------------------------------

test_d1 =  show1 (Constr 6 3 2 [Constr 2 1 0 [],Constr 1 2 2 [Constr 6 2 0 [],Constr 1 2 2 [Constr 6 1 0 [],Constr 1 2 2 
                [Constr 6 1 0 [],Constr 1 2 2 [Constr 6 2 0 [],Constr 1 2 2 [Constr 6 2 0 [],Constr 1 1 0 []]]]]]])
