module Iterpreter where

import Evaluation
import Parser
import qualified Transformation as T 

----------------------------------------------------------------------------------------------------------------------------------------------------------
-- the module Interpreter executes a programm (as a string) for three calculi: lambda(fchb), lambda(fch) and lambda(fcb),
-- and it returns all possible endpoints after a program evaluation
----------------------------------------------------------------------------------------------------------------------------------------------------------

interpretFCHB :: String -> [T.Proc']
interpretFCHB str = allEndPoints $ T.transfToProc' $  parser $ str

interpretFCH :: String -> [T.Proc']             
interpretFCH str  = allEndPoints $ T.transfToProcBtoH' $ parser $ str

interpretFCB :: String -> [T.Proc']
interpretFCB str  = allEndPoints $ T.transfToProcHtoB' $ parser $ str

----------------------------------------------------------------------------------------------------------------------------------------------------------
-- some tests
----------------------------------------------------------------------------------------------------------------------------------------------------------

test_1 =  "<n> x (<n> y (y <= (\\w -> put(x,False) ) (get x) | x <b> True ))"
test_2 = map (T.transfToProcBtoH'. T.procPrimeToProc) (allEndPoints $ (T.transfToProc' . parser $ "<n> x (<n> y (y <= (\\w -> put(x,False) ) (get x) | x <b> True ))"))
test_2a = map (T.transfToProc' . T.procPrimeToProc) (allEndPoints $ (T.transfToProcBtoH' . parser $ "<n> x (<n> y (y <= (\\w -> put(x,False) ) (get x) | x <b> True ))"))
test_3 =  "<n> x (x <= handle (\\x -> \\y -> (y True)))"
test_3a = map (T.transfToProcHtoB'. T.procPrimeToProc) (allEndPoints $ (T.transfToProc' . parser $ "<n> x (x <= handle (\\x -> \\y -> ((\\w -> (x True))(y (\\q -> q)))))"))
test_4a = map (allEndPoints . T.transfToProc' . T.procPrimeToProc) (allEndPoints $ (T.transfToProcHtoB' . parser $ "<n> x (x <= handle (\\x -> \\y -> ((\\w -> (x True))(y (\\q -> q)))))"))
test_4 =  "<n> x (x <= handle (\\x -> \\y -> ((\\w -> (x True))(y (\\q -> q)))))"
test_5 =  "<n> x (x <=  (\\b -> ((\\v -> (get b)) put(b,True))) (buffer unit))"
test_6 =  "<n> x (x <=  (\\b -> ((\\v -> (get b)) put(b, -25))) (buffer unit))"
test_7 =  "<n> x (<n> y (<n> z (<n> a (y <= (\\w -> put(x,False) ) (get x) | x <b> True | z <b> 2))) )"
test_8 =  "<n> x (<n> z ( x <= z | z <= buffer unit))"
test_9 =  map (T.transfToProcBtoH'. T.procPrimeToProc)(interpretFCHB "<n> x (<n> z ( x <= z | z <= buffer unit))")
test_10 = "<n> x (<n> z (<n> a (<n> y ( x <= a True | a <h> z ) )))"
test_11 =  map (T.transfToProcHtoB'. T.procPrimeToProc)(interpretFCHB "<n> x (<n> z (<n> a (<n> y ( x <= a True | a <h> z ) )))")
test_12 = "(<n>  x (<n> y ( <n> z ( <n> a (x <h> y | z <= x True |a <=  x False)))))"
test_13 = "(<n> p1 (<n> p2 (<n> y ( y <b> True | p1 <= get y | p2 <= (\\w -> put(y,w)) (get y) ))))"  
