module Syntax
	(
      Proc (..),
      Exp (..),
      Alts (..),
      Var,
      )
where

------------------------------------------------------------------------------------------------------
-- module Syntax contains data structures for expresions and processs of the calculus lambda-(fchb)
-- data Proc is a data structure for processes
------------------------------------------------------------------------------------------------------

data Proc = Parall Proc Proc                              -- parallel processes p_1 | p_2
          | New Var Proc                                  -- new process operator (nu x)p
          | Cell Var Exp                                  -- reference cell  x c v 
          | Thread Var Exp                                -- thread  x <= e  
          | Handle Var Var                                -- handle  y h x  
          | Usedhandle Var                                -- used handle y h .
          | Lazythread Var Exp                            -- lazy thread  x <=susp e 
          | Buffer Var Exp                                -- buffer x b e 
          | Emptybuffer Var                               -- empty buffer x b -
            deriving (Eq,Show)

------------------------------------------------------------------------------------------------------ 
-- data Exp is a data structure for expressions
------------------------------------------------------------------------------------------------------
        
data Exp 
     =  V Var                                             -- identifier 
    |   CCell                                             -- conctant cell
    |   CThread                                           -- constant thread
    |   CHandle                                           -- constant handle   
    |   CLazy                                             -- constant lazy
    |   CUnit                                             -- constant unit
    |   CBuffer                                           -- constant buffer
    |   CGet                                              -- constant get
    |   Lambda Var (Exp)                                  -- abstraction \x.e
    |   App Exp Exp                                       -- application e_1 e_2
    |   Exch Exp Exp                                      -- exch (e_1, e_2)
    |   Put Exp Exp                                       -- put (e_1, e_2)
    |   Constr Int Int Int [Exp]                          -- constructor k (x_1, x_2..)
    |   Case Exp [Alts]                                   -- case e of {alt_1 -> e_1, ... } 
        deriving (Eq,Show)      
     

data Alts = Alt Int Int Int [Var] Exp                    -- alternatives for case-expression
  deriving (Eq,Show)

--------------------------------------------------------------------------------------------------------------------------------------
-- constructors and alternatives have three integer parameters: type, number of a constructor/alternative of the same type and arity
-- the complete definition of all the conctructors and alternatives is given in the parser specification Parser.y   
--------------------------------------------------------------------------------------------------------------------------------------


type Var = String                                       -- type of identifiers




